// CL_InternComponent.c

// Handles the uart communication between the Engicam and the SWI moduls

// Include the core framework C APIs.
#include "legato.h"

#include <stdio.h>
#include <termios.h>
#include <unistd.h>


// ---------------------- defines ---------------------------------------------------------

#define UART_DEVICE "/dev/ttyHS0"
// Maximum Size for incoming String from Uart
#define BUFFER_SIZE (4096u)

#define USE_MONITOR (0u)


// ---------------------- types -----------------------------------------------------------

// ---------------------- constants -------------------------------------------------------

// ---------------------- variables -------------------------------------------------------

static int uartFd = 0;


// ---------------------- function declarations -------------------------------------------
static void initUart(const char* dev);
static void wakeupUart();
#if USE_MONITOR
static void handleUart(int fd, short events);
#else
static void* handleUart(void* object);
#endif
static void readUart(void);
static bool checkUart(void);

// ---------------------- function implementations ----------------------------------------

// +---------------------------+
// | UART                      |
// +---------------------------+

// Functions Opens and configures Uart for usage in App
static void initUart(const char* dev)
{
    int fd = 0;

    while (1)
    {
        /* O_RDWR Read/write access to the serial port.
           O_NOCTTY No terminal will control the process.
           O_NDELAY Use non-blocking I/O. */
        fd = le_tty_Open(dev, O_RDWR | O_NOCTTY | O_NDELAY);

        if (fd)
        {
            LE_INFO("UART file descriptor: %d", fd);
            break;
        }
        else
        {
            LE_INFO("Waiting for UART");
            sleep(1);
        }
    }

    /* Set baudrate to 115200. */
    le_tty_SetBaudRate(fd, LE_TTY_SPEED_115200);

    /* Set no parity, 8 data bits and 1 parity. */
    le_tty_SetFraming(fd, 'N', 8, 1);

    /* Disable the hardware flow control. */
    le_tty_SetFlowControl(fd, LE_TTY_FLOW_CONTROL_NONE);

    /* Set raw input completly non blocking. */
    le_tty_SetRaw(fd, 0, 0);

#if USE_MONITOR
    /* Setup listener. */
    le_fdMonitor_Create("Serial Port IN", /* Name for diagnostics. */
                        fd,               /* fd to monitor. */
                        handleUart,    /* Handler function. */
                        POLLIN);          /* Monitor readability. */
#else
    le_thread_Ref_t handleUartThread = le_thread_Create("UART Thread", &handleUart, (void*)(0));
    le_thread_Start(handleUartThread);
#endif
    uartFd = fd;
}

//---------------------------------------------------------------------------------------

static void wakeupUart()
{
    if (!checkUart())
    {
        LE_WARN("Cannot wakeup UART. UART is not ready");
    }
    else
    {
        LE_WARN("Waking UART up!");
        // Call tcflow to keep uart awake
        tcflow(uartFd, TCION);
    }
}

//---------------------------------------------------------------------------------------

#if USE_MONITOR
static void handleUart(int fd, short events)
{
    LE_INFO("Received UART data, %d", events);

    if (events & POLLIN)
    {
        readUart();
    }
}
#else
static void* handleUart(void* object)
{
    while (1)
    {
        usleep(500000); /* Wait 500ms before next check. */
        readUart();
    }
    return NULL;
}
#endif

//---------------------------------------------------------------------------------------

static void readUart(void)
{
    if (checkUart())
    {
        int i = 0;
        char receivedUartMessage[BUFFER_SIZE] = {0};
        char incomingChar;

        while (read(uartFd, &incomingChar, 1) == 1)
        {
            receivedUartMessage[i++] = incomingChar;
            usleep(100);
        }

        if (i > 0)
        {
            LE_INFO("Received: %s", receivedUartMessage);
        }
    }
}

//---------------------------------------------------------------------------------------

static bool checkUart(void)
{
    return (uartFd != 0);
}

//---------------------------------------------------------------------------------------
// This function is called only once on startup. Place your initialization and event registration
// here.
COMPONENT_INIT
{
    // Write in the log that this component has started.
    LE_INFO("CL_UARTOnlyTest started.");

    initUart(UART_DEVICE);

    // Configure timer to periodically wake up uart
    le_timer_Ref_t uartWakeupTimer = le_timer_Create("Wakeup Uart");
    le_timer_SetMsInterval(uartWakeupTimer, 1000);
    le_timer_SetRepeat(uartWakeupTimer, 0); // repeat indefinetly
    le_timer_SetHandler(uartWakeupTimer, &wakeupUart);
    le_timer_Start(uartWakeupTimer);
}
